/*                    Matrix.c

  Este programa bloquea un terminal de tipo Unix con una catarata
  de caracteres al estilo de "The Matrix". El objeto del programa
  fue solamente el de aprender a manejar algunas rutinas de la
  librera ncurses de Linux.
    El programa no fue diseado con la idea de que fuera portable
  y tampoco he hecho muchas pruebas con l (eso s, funciona en
  tres distribuciones de Linux bastante antiguas y en todos los
  emuladores de terminales que he podido probar en ellas, aunque
  en algunos no funciona en color).

   (c) Marc Melndez Schofield. Se permite la reproduccin
   total o parcial del cdigo de este programa, siempre que
   se haga con fines no lucrativos.


  FUNCIONAMIENTO DEL PROGRAMA

  La estructura del programa es relativamente sencilla. Los
  caracteres que caen por la pantalla son aleatorios. Los datos
  sobre las posiciones de todas las cadenas de caracteres se guardan
  en la estructura de datos ascii_char definida al principio de la
  rutina main. En ella, se almacena la fila, la columna y la longitud
  de la cadena de caracteres que va cayendo por la pantalla. La
  estructura tambin incluye algunos datos (como la anterior columna,
  por ejemplo) para ir borrando el rastro de caracteres que va dejando.
    Al comienzo del programa se crea un array de datos del tipo
  ascii_char. Se inician todas las cadenas en la ltima columna 
  (max_col), que, segn he podido comprobar, no se muestra en pantalla.
  Luego se definen los colores.
    Despus, en el bucle principal del programa, simplemente se
  actualizan las posiciones de las cadenas y se refresca la pantalla.
*/


#include <curses.h>
#include <stdlib.h>
#include <unistd.h>

#define U_SLEEP_TIME 500
#define CHAIN_NUM 60

void main ()
{
  struct ascii_char
  {
  int row, col, old_col;
  int speed, speed_counter;
  int chain_length;
  int char_value, old_char_value;
  } chars[CHAIN_NUM];

  int max_row, max_col;
  int i;

  initscr();
  getmaxyx(stdscr, max_row, max_col);

  // Initialize the ascii_char array

  for(i = 0; i < CHAIN_NUM; i++)
    {
      chars[i].row = 1 + (int) ((double) max_row*rand()/(RAND_MAX+1.0));
      chars[i].col = max_col; //1 + (int) ((double) max_col*rand()/(RAND_MAX+1.0));
      chars[i].speed = (int) (4.0*rand()/(RAND_MAX+1.0));
      chars[i].chain_length = 2 + (int) (12.0*rand()/(RAND_MAX+1.0));
      chars[i].char_value = 50 + (int) (60.0*rand()/(RAND_MAX+1.0));
      chars[i].old_char_value = ' ';
    }

  start_color();
  if(can_change_color()) init_color(COLOR_WHITE, 800, 1000, 800);
  init_pair(1, COLOR_GREEN, COLOR_BLACK);
  init_pair(2, COLOR_WHITE, COLOR_BLACK);
  init_pair(3, COLOR_BLACK, COLOR_BLACK);

  for(;;)
    {
      for(i = 0; i < CHAIN_NUM; i++)
	{
	  if(chars[i].speed_counter == 0)
	    {
	      if(chars[i].row > max_row)
		{
		  chars[i].row = 0;
		  chars[i].old_col = chars[i].col;
		  chars[i].col = 1 + (int) ((double) max_col*rand()/
					    (RAND_MAX+1.0));

		  attron(COLOR_PAIR(1));
		  mvaddch(max_row, chars[i].old_col, chars[i].char_value);
		  mvaddch(max_row - 1, chars[i].old_col,
			  chars[i].old_char_value);
		  attroff(COLOR_PAIR(1));

		  chars[i].char_value=50 + (int) (60.0*rand()/(RAND_MAX+1.0));

		  attron(COLOR_PAIR(2));
		  mvaddch(chars[i].row, chars[i].col,
			  chars[i].char_value | A_BOLD);
		  attroff(COLOR_PAIR(2));
		  chars[i].row++;
		}
	      else
		{
		  attron(COLOR_PAIR(1));
		  mvaddch(chars[i].row - 1, chars[i].col,
			  chars[i].char_value | A_BOLD);
		  if(chars[i].row > 1)
		    mvaddch(chars[i].row - 2, chars[i].col,
			    chars[i].old_char_value);
		  attroff(COLOR_PAIR(1));

		  chars[i].old_char_value = chars[i].char_value;
		  chars[i].char_value=50 + (int) (60.0*rand()/(RAND_MAX+1.0));

		  attron(COLOR_PAIR(2));
		  mvaddch(chars[i].row, chars[i].col,
			  chars[i].char_value | A_BOLD);
		  attroff(COLOR_PAIR(2));

		  if(chars[i].row > chars[i].chain_length)
		    {
		      attron(COLOR_PAIR(3));
		      mvaddch(chars[i].row - chars[i].chain_length - 1,
			      chars[i].col, ' ');
		      attroff(COLOR_PAIR(3));
		    }
		  else
		    {
		      attron(COLOR_PAIR(3));
		      mvaddch(max_row - chars[i].chain_length 
			      + chars[i].row - 1, chars[i].old_col, ' ');
		      attroff(COLOR_PAIR(3));
		    }
		  chars[i].row++;
		}
	      chars[i].speed_counter = chars[i].speed;
	    }
	  else
	    {
	      chars[i].speed_counter = chars[i].speed_counter - 1;
	    }
	}
      usleep(U_SLEEP_TIME);
      refresh();
    }

  endwin();
}
